/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.5 Funktionen Ganglinie
 * Copyright (C) 2011-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.iav.gllib.gllib.domain;

import java.time.Duration;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.Set;

/**
 * Repräsentation der Lernparameter eines Ereignistyps.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class EreignisTypLernParameter {

    public static final EreignisTypLernParameter NULL = new EreignisTypLernParameter();

    private Set<EreignisTyp> ausschlussliste = new LinkedHashSet<>();
    private GanglinieTyp ganglinieTyp = GanglinieTyp.ABSOLUT;
    private Duration vergleichsSchrittweite = Duration.ofSeconds(300);
    private int maximalerAbstand = 50;
    private Set<EreignisTyp> bezugsTypen = new LinkedHashSet<>();
    private int maximalerMatchingFehler = 50;
    private Duration matchingSchrittweite = Duration.ofSeconds(300);
    private Duration matchingIntervallVor = Duration.ofSeconds(21600);
    private Duration matchingIntervallNach = Duration.ofSeconds(21600);
    private int maximalerWichtungsFaktor = 10;
    private ApproximationTyp darstellungsVerfahren = ApproximationTyp.POLYLINE;
    private long maxGanglinien = 5;

    public boolean isNull() {
        return maxGanglinien <= 0;
    }

    public Collection<EreignisTyp> getAusschlussliste() {
        return Collections.unmodifiableCollection(ausschlussliste);
    }

    public Set<EreignisTyp> getBezugsTypen() {
        return bezugsTypen;
    }

    public ApproximationTyp getDarstellungsVerfahren() {
        return darstellungsVerfahren;
    }

    public GanglinieTyp getGanglinieTyp() {
        return ganglinieTyp;
    }

    public Duration getMatchingIntervallNach() {
        return matchingIntervallNach;
    }

    public Duration getMatchingIntervallVor() {
        return matchingIntervallVor;
    }

    public Duration getMatchingSchrittweite() {
        return matchingSchrittweite;
    }

    public long getMaxGanglinien() {
        return maxGanglinien;
    }

    public int getMaximalerAbstand() {
        return maximalerAbstand;
    }

    public int getMaximalerMatchingFehler() {
        return maximalerMatchingFehler;
    }

    public int getMaxWichtungsFaktor() {
        return maximalerWichtungsFaktor;
    }

    public Duration getVergleichsSchrittweite() {
        return vergleichsSchrittweite;
    }

    public void addAusschluss(EreignisTyp typ) {
        this.ausschlussliste.add(typ);
    }

    public void addBezug(EreignisTyp typ) {
        this.bezugsTypen.add(typ);
    }

    public void setDarstellungsVerfahren(ApproximationTyp darstellungsVerfahren) {
        this.darstellungsVerfahren = darstellungsVerfahren;
    }

    public void setGanglinieTyp(GanglinieTyp ganglinieTyp) {
        this.ganglinieTyp = ganglinieTyp;
    }

    public void setMatchingIntervallNach(Duration matchingIntervallNach) {
        this.matchingIntervallNach = matchingIntervallNach;
    }

    public void setMatchingIntervallVor(Duration matchingIntervallVor) {
        this.matchingIntervallVor = matchingIntervallVor;
    }

    public void setMatchingSchrittweite(Duration matchingSchrittweite) {
        this.matchingSchrittweite = matchingSchrittweite;
    }

    public void setMaxGanglinien(long maxGanglinien) {
        this.maxGanglinien = maxGanglinien;
    }

    public void setMaximalerAbstand(int maximalerAbstand) {
        this.maximalerAbstand = maximalerAbstand;
    }

    public void setMaximalerMatchingFehler(int maximalerMatchingFehler) {
        this.maximalerMatchingFehler = maximalerMatchingFehler;
    }

    public void setMaximalerWichtungsFaktor(int maximalerWichtungsFaktor) {
        this.maximalerWichtungsFaktor = maximalerWichtungsFaktor;
    }

    public void setVergleichsSchrittweite(Duration vergleichsSchrittweite) {
        this.vergleichsSchrittweite = vergleichsSchrittweite;
    }

}
