/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.5 Funktionen Ganglinie
 * Copyright (C) 2011-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.gllib.gllib.domain;

import java.time.LocalDateTime;

import de.bsvrz.iav.gllib.gllib.shared.LocalDateTimeKonverter;

/**
 * Intervall für das eine Prognose ermittelt werden soll, begrenzt durch einen
 * Anfangs- und Endzeitpunkt.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public final class PrognoseIntervall {

    private final LocalDateTime von;
    private final LocalDateTime bis;

    private PrognoseIntervall(LocalDateTime von, LocalDateTime bis) {
        this.von = von;
        this.bis = bis;
    }

    public static PrognoseIntervall of(long von, long bis) {
        return new PrognoseIntervall(LocalDateTimeKonverter.fromMillis(von), LocalDateTimeKonverter.fromMillis(bis));
    }

    public static PrognoseIntervall of(LocalDateTime von, LocalDateTime bis) {
        return new PrognoseIntervall(von, bis);
    }

    public LocalDateTime getVon() {
        return von;
    }

    public LocalDateTime getBis() {
        return bis;
    }
    
    public boolean isValid() {
        return bis.isAfter(von);
    }

    @Override
    public String toString() {
        return "PrognoseIntervall [von=" + von + ", bis=" + bis + "]";
    }

}
