/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.5 Funktionen Ganglinie
 * Copyright (C) 2011-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.gllib.gllib.domain.comm;

import java.time.Duration;
import java.util.List;
import java.util.concurrent.TimeoutException;
import java.util.function.Consumer;

import de.bsvrz.iav.gllib.gllib.domain.GanglinienAnfrager;
import de.bsvrz.iav.gllib.gllib.domain.GanglinienException;
import de.bsvrz.iav.gllib.gllib.domain.MqGanglinie;

/**
 * Schnittstelle für den Zugriff auf den GanglinienSpeicher über eine
 * Anfrage/Antwort-Schnittstelle.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 *
 */
public interface GanglinienSpeicherClient {
	
    
    GanglinienAnfrager getAnfrager();

    String getAnfrageId();

    List<MqGanglinie> liesGanglinien(GlLesenAnfrage anfrage, Duration timeout)
            throws TimeoutException, GanglinienException;

    void liesGanglinien(GlLesenAnfrage anfrage, Consumer<GlSpeicherAntwort> antwort,
            Consumer<GlSpeicherFehlerAntwort> fehlerAntwort);

    void liesGanglinien(GlLesenAnfrage anfrage, Consumer<GlSpeicherAntwort> speicherAntwort,
            Consumer<GlSpeicherFehlerAntwort> speicherFehlerAntwort, Duration timeout);

    void schreibeGanglinien(GlSchreibenAnfrage anfrage, Consumer<GlSpeicherAntwort> antwort,
            Consumer<GlSpeicherFehlerAntwort> fehlerAntwort);

    void schreibeGanglinien(GlSchreibenAnfrage anfrage, Consumer<GlSpeicherAntwort> antwort,
            Consumer<GlSpeicherFehlerAntwort> fehlerAntwort, Duration timeout);

    void loescheGanglinien(GlLoeschenAnfrage anfrage, Consumer<GlSpeicherAntwort> antwort,
            Consumer<GlSpeicherFehlerAntwort> fehlerAntwort);

    void loescheGanglinien(GlLoeschenAnfrage anfrage, Consumer<GlSpeicherAntwort> antwort,
            Consumer<GlSpeicherFehlerAntwort> fehlerAntwort, Duration timeout);

    List<MqGanglinie> erzeugeGanglinien(GlAnlegenAnfrage anfrage, Duration timeout)
            throws TimeoutException, GanglinienException;

    void erzeugeGanglinien(GlAnlegenAnfrage anfrage, Consumer<GlSpeicherAntwort> antwort,
            Consumer<GlSpeicherFehlerAntwort> fehlerAntwort);

    void erzeugeGanglinien(GlAnlegenAnfrage anfrage, Consumer<GlSpeicherAntwort> antwort,
            Consumer<GlSpeicherFehlerAntwort> fehlerAntwort, Duration timeout);

    List<MqGanglinie> aktualisiereGanglinien(GlAktualisierenAnfrage anfrage, Duration timeout)
            throws TimeoutException, GanglinienException;

    void aktualisiereGanglinien(GlAktualisierenAnfrage anfrage, Consumer<GlSpeicherAntwort> antwort,
            Consumer<GlSpeicherFehlerAntwort> fehlerAntwort);

    void aktualisiereGanglinien(GlAktualisierenAnfrage anfrage, Consumer<GlSpeicherAntwort> antwort,
            Consumer<GlSpeicherFehlerAntwort> fehlerAntwort, Duration timeout);
}
