/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.5 Funktionen Ganglinie
 * Copyright (C) 2011-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.gllib.gllib.portals;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.iav.gllib.gllib.domain.EreignisTyp;
import de.bsvrz.iav.gllib.gllib.domain.comm.GlProgAnfrage;
import de.bsvrz.iav.gllib.gllib.domain.comm.PrognoseAnfrage;
import de.bsvrz.iav.gllib.gllib.shared.LocalDateTimeKonverter;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * 
 * Implementierung eines Senders zum Versenden von Anfragen an die
 * Ganglinienprognose über die Datenverteilerschnittstelle.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class DavPrognoseAnfrageSender {

    private static final Debug LOGGER = Debug.getLogger();

    private static ClientSenderInterface sender = new ClientSenderInterface() {

        @Override
        public boolean isRequestSupported(SystemObject object, DataDescription dataDescription) {
            return false;
        }

        @Override
        public void dataRequest(SystemObject object, DataDescription dataDescription, byte state) {
            // Sendesteuerung wird nicht ausgewertet
        }
    };

    private ClientDavInterface dav;
    private AttributeGroup anfrageAtg;
    private Aspect anfrageAsp;

    public DavPrognoseAnfrageSender(ClientDavInterface dav) {
        this.dav = dav;
        anfrageAtg = dav.getDataModel().getAttributeGroup("atg.prognoseGanglinienAnfrage");
        anfrageAsp = dav.getDataModel().getAspect("asp.anfrage");
        try {
            dav.subscribeSender(sender, dav.getLocalConfigurationAuthority(),
                    new DataDescription(anfrageAtg, anfrageAsp), SenderRole.sender());
        } catch (OneSubscriptionPerSendData e) {
            LOGGER.warning(e.getLocalizedMessage());
        }
    }

    public void sendeAnfrage(GlProgAnfrage anfrage) {
        try {
            dav.sendData(konvertiereAnfrage(anfrage));
        } catch (DataNotSubscribedException | SendSubscriptionNotConfirmed e) {
            LOGGER.warning(e.getLocalizedMessage());
        }
    }

    private ResultData konvertiereAnfrage(GlProgAnfrage anfrage) {

        Data daten = dav.createData(anfrageAtg);

        daten.getReferenceValue("absenderId").setSystemObject(dav.getLocalApplicationObject());
        daten.getTextValue("AbsenderZeichen").setText(anfrage.getAbsenderZeichen());

        Array feld = daten.getArray("PrognoseGanglinienAnfrage");
        feld.setLength(anfrage.getAnfragen().size());
        int i = 0;
        for (final PrognoseAnfrage prognoseAnfrage : anfrage.getAnfragen()) {
            getDaten(prognoseAnfrage, feld.getItem(i++));
        }

        return new ResultData(dav.getLocalConfigurationAuthority(), new DataDescription(anfrageAtg, anfrageAsp),
                dav.getTime(), daten);
    }

    private void getDaten(PrognoseAnfrage prognoseAnfrage, final Data daten) {

        daten.getReferenceValue("Messquerschnitt").setSystemObjectPid(prognoseAnfrage.getMessQuerschnitt().getPid());
        daten.getTimeValue("ZeitpunktPrognoseBeginn")
                .setMillis(LocalDateTimeKonverter.toMillis(prognoseAnfrage.getPrognoseZeitraum().getVon()));
        daten.getTimeValue("ZeitpunktPrognoseEnde")
                .setMillis(LocalDateTimeKonverter.toMillis(prognoseAnfrage.getPrognoseZeitraum().getBis()));
        daten.getScaledValue("Überprüfungsintervall").set(prognoseAnfrage.getPruefIntervall().toMillis());
        daten.getScaledValue("Aktualisierungsschwelle").set(prognoseAnfrage.getSchwelle());
        daten.getScaledValue("Aktualisierungsintervall").set(prognoseAnfrage.getSendeIntervall().toMillis());

        if (prognoseAnfrage.isNurLangfristigeAuswahl()) {
            daten.getUnscaledValue("NurLangfristigeAuswahl").setText("Ja");
        } else {
            daten.getUnscaledValue("NurLangfristigeAuswahl").setText("Nein");
        }

        if (prognoseAnfrage.isZyklischePrognose()) {
            daten.getUnscaledValue("ZyklischePrognose").setText("Ja");
        } else {
            daten.getUnscaledValue("ZyklischePrognose").setText("Nein");
        }

        Array feld = daten.getArray("EreignisTyp");
        feld.setLength(prognoseAnfrage.getEreignisTypen().size());
        int i = 0;
        for (final EreignisTyp et : prognoseAnfrage.getEreignisTypen()) {
            feld.getItem(i++).asReferenceValue().setSystemObjectPid(et.getPid());
        }
    }
}
